import os
import requests
import socket
from python_settings import settings 
from ipaddress import ip_address, IPv4Address, IPv6Address
from pathlib import Path

BASE_DIR = Path(__file__).resolve().parent


def get_html():
    with open(os.path.join(BASE_DIR, "templates", "Forbidden403Page.html"), 'r') as file:  # r to open file in READ mode
        html_as_string = file.read()
    return html_as_string


class RuleService:
    def __init__(self):
        self.api_url = settings.API_URL
        print("settings",settings.TOKEN)
        if settings.TOKEN is None or settings.TOKEN == "":
            raise Exception("You have set your API KEY. see: https://protectremote.com/sss")
        self.token = settings.TOKEN

    def post_rules(self):
        try:
            response = requests.get(self.api_url, headers={"X-Token": self.token,"Accept": "application/json"})
            response.raise_for_status()
            response_json = response.json()
            print("response_json",response_json)
            return response_json
        except requests.exceptions.HTTPError as errh:
            print ("Http Error:",errh)
        except requests.exceptions.ConnectionError as errc:
            print ("Error Connecting:",errc)
        except requests.exceptions.Timeout as errt:
            print ("Timeout Error:",errt)
        except requests.exceptions.RequestException as err:
            print ("OOps: Something Else",err)
        except Exception as e:
            return e


def get_local_ip():
    hostname = socket.gethostname()
    IPAddr = socket.gethostbyname(hostname)
    return IPAddr


def has_ip_access()->bool:
    # Get local IP address

    if settings.DEBUG_MODE:
        IPAddr = settings.DEBUG_MODE_IP_ADDRESS
        print("Local ip address:", IPAddr)
    else:
        IPAddr =  get_local_ip()

    # get api rules from settings
    data = settings.RULES_DATA


    if "rules" not in data:
        print("Cachede data mevcut değil")
        return False

    rules = data["rules"]
    hasAccess = False
    for rule in rules:
        # check first ipv4
        if type(ip_address(IPAddr)) == IPv4Address:
            if "ipAddressV4List" in rule and IPAddr in rule["ipAddressV4List"]:
                    hasAccess= True
        elif type(ip_address(IPAddr)) == IPv6Address:
            if "ipAddressV6List" in rule and IPAddr in rule["ipAddressV6List"]:
                    hasAccess= True
        else:
            raise Exception("ipv4 veya ipv6 ip adresi göndermelisiniz")
    return hasAccess


    






from protectremote.main import RuleService
from python_settings import settings 


def update_request_data():
    try:
        service = RuleService()
        data = service.post_rules()
        settings.RULES_DATA = data
        print('Data updated...', settings.RULES_DATA)
    except Exception as e:
        print("data couldnt updated", e)
    


import os
os.environ["SETTINGS_MODULE"] = 'protectremote.settings'

from protectremote.middleware import ProtechRemoteMiddleware, pr_access
from python_settings import settings
from apscheduler.schedulers.background import BackgroundScheduler
from protectremote.job import update_request_data
import copy



def set_token(token: str):
    settings.TOKEN = token
    print("new token",   settings.TOKEN)
    
def set_debug(ip_address: str):
    settings.DEBUG_MODE = True
    settings.DEBUG_MODE_IP_ADDRESS = ip_address

def set_request_interval(seconds: int):
    settings.REQUEST_INTERVAL_IN_SECONDS = seconds

def set_render_html_page(active: bool):
    settings.RENDER_HTML_PAGE = active


scheduler = BackgroundScheduler()
job = scheduler.add_job(update_request_data, 'interval', seconds=15)

scheduler.start()


__all__ = [
    "pr_access",
    "set_token",
    "set_debug",
    "set_render_html_page",
    "ProtechRemoteMiddleware"
]


from werkzeug.wrappers import Request, Response
from .main import has_ip_access, get_html
from functools import wraps
from flask import make_response, jsonify, render_template
from python_settings import settings 
from pathlib import Path

BASE_DIR = Path(__file__).resolve().parent



class ProtechRemoteMiddleware:

    def __init__(self, app):
        self.app = app

    def __call__(self, environ, start_response):
        request = Request(environ)
        
        if not has_ip_access():
            # TODO: response type ( html or json)
            if settings.RENDER_HTML_PAGE:
                res = Response(get_html(), mimetype='text/html', status=401)
            else:
                res = Response(settings.RESTRICTED_MESSAGE, mimetype='text/plain', status=401)
            return res(environ, start_response)
        return self.app(environ, start_response)
    

def pr_access():
    def _pr_access(f):
        @wraps(f)
        def __pr_access(*args, **kwargs):
            # just do here everything what you need
            if not has_ip_access():
                if settings.RENDER_HTML_PAGE:
                    return make_response(get_html(), 401)
                else:
                    return make_response(settings.RESTRICTED_MESSAGE, 401)
            result = f(*args, **kwargs)
            return result
        return __pr_access
    return _pr_access



API_URL = "https://test-packapi.protectremote.com/rules"
REQUEST_INTERVAL_IN_SECONDS=15
DEBUG_MODE_IP_ADDRESS = ""
DEBUG_MODE=False
RENDER_HTML_PAGE=True
TOKEN = ""
RESTRICTED_MESSAGE = "PROTECT_REMOTE_ACCESS_DENIED"
RULES_DATA= []


import os
from setuptools import setup, find_packages
from importlib.machinery import SourceFileLoader

version = (
    SourceFileLoader("protectremote.version", os.path.join("protectremote", "version.py")).load_module().VERSION
)

setup(
    name='protectremote',
    version=version,
    packages=find_packages(exclude=['tests*']),
    license='MIT',
    description='Next-gen security solution for remote workers',
    long_description_content_type='text/markdown',
    long_description=open('README.md').read(),
    install_requires=['python-settings',"flask", "apscheduler", "python-dotenv"],
    author='Protect Remote',
    author_email='sinan.bozkus@gmail.com',
    keywords=['security'],
    include_package_data=True,
     package_data = {
        '': ['*.html', '*.css'],
    },
)